import 'package:global_state/src/controller.dart';
import 'package:global_state/src/structures.dart';

import 'base.dart';

/// Benchmarks for the GSMap implementation.
///
/// Enter 'flutter test .\benchmark\notifier_benchmark.dart' in console
/// from the root folder of the project to run the benchmarks.

main() => runGSNotifierBenchmarks();

runGSNotifierBenchmarks() {
  print('');

  const readCounts = [1, 3, 5, 10, 100, 1000, 10000, 100000];
  for (var i in readCounts) {
    print('\n-------Read benchmarks $i Values-----------------------');
    readBenchmarks(i);
  }
  print('-----------------------------------------------------');
  print(
      '-------------End GSNotifier Read BenchMarks---------------------------------');
  print('-----------------------------------------------------\n');

  const writeCounts = [1, 3, 5, 10, 100, 1000, 10000, 100000];
  for (var i in writeCounts) {
    print('\n-------Write benchmarks $i Values-----------------------');
    runWriteBenchmark(i, 'Write benchmark $i values');
    print('-----------------------------------------------------\n');
  }

  print(
      '-------------End GSNotifier Write Benchmarks--------------------------------');
  print('------------------$output-----------------------------------\n');
}

void readBenchmarks([int numberOfValues = 1000]) {
  Map data = createMapWithValues(numberOfValues);
  Iterable keys = data.keys.toList();
  runReadBenchmark(data, keys, 'Unordered keys read benchmark');
}

runReadBenchmark(Map data, Iterable keys,
    [String benchmarkHeadLine = 'Running Benchmark']) {
  print('\n$benchmarkHeadLine\n');
  MockListener mockElement = MockListener();

  GSController.debugReset();

  Map readMap = GSMap.of(data);
  var obsTime =
      benchmarkFunction(() => plainRead(readMap, keys), 'ObservadMap warm up');

  readMap = Map.of(data);
  var refTime = benchmarkFunction(
      () => plainRead(readMap, keys), 'LinkedHashMap warm up');

  print('');

  readMap = GSMap.of(data);
  obsTime = benchmarkFunction(() => plainRead(readMap, keys), 'ObservadMap');

  readMap = Map.of(data);
  refTime = benchmarkFunction(() => plainRead(readMap, keys), 'LinkedHashMap');

  print('----Using ${GSController}----');

  readMap = GSMap.of(data);
  GSController.startListening(mockElement);
  var obsTimeListening = benchmarkFunction(
      () => plainRead(readMap, keys), 'ObservadMap while listening');
  GSController.stopListening();

  readMap = GSMap.of(data);
  addNotifierSubscriptions(readMap); // loads the controller with subscriptions
  GSController.startListening(mockElement);
  var obsTimeListening2 = benchmarkFunction(() => plainRead(readMap, keys),
      'ObservadMap while listening with filled controller');
  GSController.stopListening();
  GSController.debugReset();

  readMap = GSMap.of(data);
  var obsTimeListeningCycle = benchmarkFunction(() {
    GSController.startListening(mockElement);
    plainRead(readMap, keys);
    GSController.stopListening();
  }, 'ObservadMap complete listening cycle');

  readMap = GSMap.of(data);
  addNotifierSubscriptions(readMap);
  var obsTimeListeningCycle2 = benchmarkFunction(() {
    GSController.startListening(mockElement);
    plainRead(readMap, keys);
    GSController.stopListening();
  }, 'ObservadMap complete listening cycle with filled controller');

  GSController.debugReset();

  benchmarkFunction(() {
    for (var k in keys) output = k;
  }, 'Iterate keys');

  print(
      '\nGSNotifier access time overhead x${(obsTime / refTime).toStringAsFixed(2)}');
  print(
      'GSNotifier access time overhead while listening x${(obsTimeListening / refTime).toStringAsFixed(2)}');
  print(
      'GSNotifier access time overhead while listening with controller Standard filled x${(obsTimeListening2 / refTime).toStringAsFixed(2)}');
  // print(
  //     'GSNotifier access time overhead while listening Light x${(obsTimeLightListening / refTime).toStringAsFixed(2)}');
  // print(
  //     'GSNotifier access time overhead while listening with controller Light filled x${(obsTimeLightListening2 / refTime).toStringAsFixed(2)}');
  print('----Full cycle ratios----');
  print(
      'Complete listen cycle access time overhead x${(obsTimeListeningCycle / refTime).toStringAsFixed(2)}');
  print(
      'Complete listen cycle access time overhead with controller Standard filled x${(obsTimeListeningCycle2 / refTime).toStringAsFixed(2)}');
  // print(
  //     'Complete listen cycle access time overhead Light x${(obsTimeLightListeningCycle / refTime).toStringAsFixed(2)}');
  // print(
  //     'Complete listen cycle access time overhead with controller Light filled x${(obsTimeLightListeningCycle2 / refTime).toStringAsFixed(2)}');
}

runWriteBenchmark([
  int writeCount = 10000,
  String benchmarkHeadLine = 'Running Benchmark',
]) {
  print('\n${benchmarkHeadLine.toUpperCase()}\n');

  Map writeMap = GSMap();
  var obsTime = benchmarkFunction(
      () => plainWrite(writeMap, writeCount), 'ObservadMap 1 warm up');

  writeMap = Map();
  var refTime = benchmarkFunction(
      () => plainWrite(writeMap, writeCount), 'LinkedHashMap 1 warm up');

  writeMap = GSMap();
  obsTime =
      benchmarkFunction(() => plainWrite(writeMap, writeCount), 'ObservadMap');

  writeMap = Map();
  refTime = benchmarkFunction(
      () => plainWrite(writeMap, writeCount), 'LinkedHashMap');

  benchmarkFunction(() {
    for (var i = 0; i < writeCount; i++);
  }, 'Iterate keys');

  print(
      '\nGSNotifier write time overhead x${(obsTime / refTime).toStringAsFixed(2)}');
  print('');
}

plainWrite(Map map, int n) {
  for (var i = 0; i < n; i++) {
    map[i] = i;
  }
}
